@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2023, University of Amsterdam
+    Copyright (c)  1985-2024, University of Amsterdam
                               VU University Amsterdam
                               SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -184,8 +184,8 @@ load_script_files([OsFile|More]) :-
                              file_errors(fail)
                            ])
     ->  asserta(loaded_init_file(script, Path)),
-        load_files(user:Path, []),
-        load_files(More)
+        load_files(user:Path),
+        load_files(user:More)
     ;   throw(error(existence_error(script_file, File), _))
     ).
 
@@ -323,12 +323,8 @@ thread_initialization(Goal) :-
     { '$make_alias'(AliasChars, Alias) }.
 
 path_sep -->
-    { current_prolog_flag(windows, true)
-    },
-    !,
-    [;].
-path_sep -->
-    [:].
+    { current_prolog_flag(path_sep, Sep) },
+    [Sep].
 
 '$string'([]) --> [].
 '$string'([H|T]) --> [H], '$string'(T).
@@ -762,9 +758,10 @@ run_init_goal(Goal, Ctx) :-
 %   toplevel.
 
 init_debug_flags :-
-    once(print_predicate(_, [print], PrintOptions)),
     Keep = [keep(true)],
-    create_prolog_flag(answer_write_options, PrintOptions, Keep),
+    create_prolog_flag(answer_write_options,
+                       [ quoted(true), portray(true), max_depth(10),
+                         spacing(next_argument)], Keep),
     create_prolog_flag(prompt_alternatives_on, determinism, Keep),
     create_prolog_flag(toplevel_extra_white_line, true, Keep),
     create_prolog_flag(toplevel_print_factorized, false, Keep),
@@ -1298,7 +1295,7 @@ subst_chars([H|T]) -->
     ->  DetOrChp = true
     ;   DetOrChp = Chp
     ),
-    call_expand_answer(Bindings, NewBindings),
+    call_expand_answer(Goal, Bindings, NewBindings),
     (    \+ \+ write_bindings(NewBindings, Vars, Delays, DetOrChp)
     ->   !
     ).
@@ -1341,19 +1338,21 @@ write_bindings(Bindings, ResidueVars, Delays, DetOrChp) :-
     '$current_typein_module'(TypeIn),
     translate_bindings(Bindings, Bindings1, ResidueVars, TypeIn:Residuals),
     omit_qualifier(Delays, TypeIn, Delays1),
-    name_vars(Bindings1, t(Residuals, Delays1)),
     write_bindings2(Bindings1, Residuals, Delays1, DetOrChp).
 
 write_bindings2([], Residuals, Delays, _) :-
     current_prolog_flag(prompt_alternatives_on, groundness),
     !,
+    name_vars([], t(Residuals, Delays)),
     print_message(query, query(yes(Delays, Residuals))).
 write_bindings2(Bindings, Residuals, Delays, true) :-
     current_prolog_flag(prompt_alternatives_on, determinism),
     !,
+    name_vars(Bindings, t(Residuals, Delays)),
     print_message(query, query(yes(Bindings, Delays, Residuals))).
 write_bindings2(Bindings, Residuals, Delays, Chp) :-
     repeat,
+        name_vars(Bindings, t(Residuals, Delays)),
         print_message(query, query(more(Bindings, Delays, Residuals))),
         get_respons(Action, Chp),
     (   Action == redo
@@ -1380,6 +1379,7 @@ write_bindings2(Bindings, Residuals, Delays, Chp) :-
 
 name_vars(Bindings, Term) :-
     current_prolog_flag(toplevel_name_variables, true),
+    answer_flags_imply_numbervars,
     !,
     '$term_multitons'(t(Bindings,Term), Vars),
     name_vars_(Vars, Bindings, 0),
@@ -1422,6 +1422,21 @@ in_vars(Name, Name) :- !.
 in_vars(Names, Name) :-
     '$member'(Name, Names).
 
+%!  answer_flags_imply_numbervars
+%
+%   True when the answer will be  written recognising '$VAR'(N). If this
+%   is not the case we should not try to name the variables.
+
+answer_flags_imply_numbervars :-
+    current_prolog_flag(answer_write_options, Options),
+    numbervars_option(Opt),
+    memberchk(Opt, Options),
+    !.
+
+numbervars_option(portray(true)).
+numbervars_option(portrayed(true)).
+numbervars_option(numbervars(true)).
+
 %!  residual_goals(:NonTerminal)
 %
 %   Directive that registers NonTerminal as a collector for residual
@@ -1845,7 +1860,8 @@ answer_respons(0'*, Chp, show_again) :-
     !,
     print_last_chpoint(Chp).
 answer_respons(Char, _, show_again) :-
-    print_predicate(Char, Pred, Options),
+    current_prolog_flag(answer_write_options, Options0),
+    print_predicate(Char, Pred, Options0, Options),
     !,
     print_message(query, if_tty(['~w'-[Pred]])),
     set_prolog_flag(answer_write_options, Options).
@@ -1856,15 +1872,55 @@ answer_respons(-1, _, show_again) :-
 answer_respons(Char, _, again) :-
     print_message(query, no_action(Char)).
 
-print_predicate(0'w, [write], [ quoted(true),
-                                spacing(next_argument)
-                              ]).
-print_predicate(0'p, [print], [ quoted(true),
-                                portray(true),
-                                max_depth(10),
-                                spacing(next_argument)
-                              ]).
+%!  print_predicate(+Code, -Change, +Options0, -Options) is semidet.
+%
+%   Modify  the  `answer_write_options`  value  according  to  the  user
+%   command.
 
+print_predicate(0'w, [write], Options0, Options) :-
+    edit_options([-portrayed(true),-portray(true)],
+                 Options0, Options).
+print_predicate(0'p, [print], Options0, Options) :-
+    edit_options([+portrayed(true)],
+                 Options0, Options).
+print_predicate(0'+, [Change], Options0, Options) :-
+    (   '$select'(max_depth(D0), Options0, Options1)
+    ->  D is D0*10,
+        format(string(Change), 'max_depth(~D)', [D]),
+        Options = [max_depth(D)|Options1]
+    ;   Options = Options0,
+        Change = 'no max_depth'
+    ).
+print_predicate(0'-, [Change], Options0, Options) :-
+    (   '$select'(max_depth(D0), Options0, Options1)
+    ->  D is max(1, D0//10),
+        Options = [max_depth(D)|Options1]
+    ;   D = 10,
+        Options = [max_depth(D)|Options0]
+    ),
+    format(string(Change), 'max_depth(~D)', [D]).
+
+edit_options([], Options, Options).
+edit_options([H|T], Options0, Options) :-
+    edit_option(H, Options0, Options1),
+    edit_options(T, Options1, Options).
+
+edit_option(-Term, Options0, Options) =>
+    (   '$select'(Term, Options0, Options)
+    ->  true
+    ;   Options = Options0
+    ).
+edit_option(+Term, Options0, Options) =>
+    functor(Term, Name, 1),
+    functor(Var, Name, 1),
+    (   '$select'(Var, Options0, Options1)
+    ->  Options = [Term|Options1]
+    ;   Options = [Term|Options0]
+    ).
+
+%!  print_last_chpoint(+Chp) is det.
+%
+%   Print the last choicepoint when an answer is nondeterministic.
 
 print_last_chpoint(Chp) :-
     current_predicate(print_last_choice_point/0),
@@ -1886,21 +1942,30 @@ print_last_chpoint_(Chp) :-
 :- user:multifile(expand_query/4).
 
 call_expand_query(Goal, Expanded, Bindings, ExpandedBindings) :-
-    user:expand_query(Goal, Expanded, Bindings, ExpandedBindings),
-    !.
-call_expand_query(Goal, Expanded, Bindings, ExpandedBindings) :-
-    toplevel_variables:expand_query(Goal, Expanded, Bindings, ExpandedBindings),
-    !.
-call_expand_query(Goal, Goal, Bindings, Bindings).
+    (   '$replace_toplevel_vars'(Goal, Expanded0, Bindings, ExpandedBindings0)
+    ->  true
+    ;   Expanded0 = Goal, ExpandedBindings0 = Bindings
+    ),
+    (   user:expand_query(Expanded0, Expanded, ExpandedBindings0, ExpandedBindings)
+    ->  true
+    ;   Expanded = Expanded0, ExpandedBindings = ExpandedBindings0
+    ).
 
 
-:- user:dynamic(expand_answer/2).
-:- user:multifile(expand_answer/2).
+:- dynamic
+    user:expand_answer/2,
+    prolog:expand_answer/3.
+:- multifile
+    user:expand_answer/2,
+    prolog:expand_answer/3.
 
-call_expand_answer(Goal, Expanded) :-
-    user:expand_answer(Goal, Expanded),
+call_expand_answer(Goal, BindingsIn, BindingsOut) :-
+    (   prolog:expand_answer(Goal, BindingsIn, BindingsOut)
+    ->  true
+    ;   user:expand_answer(BindingsIn, BindingsOut)
+    ->  true
+    ;   BindingsOut = BindingsIn
+    ),
+    '$save_toplevel_vars'(BindingsOut),
     !.
-call_expand_answer(Goal, Expanded) :-
-    toplevel_variables:expand_answer(Goal, Expanded),
-    !.
-call_expand_answer(Goal, Goal).
+call_expand_answer(_, Bindings, Bindings).
