@@ -82,7 +82,7 @@ PLX_ASIS(void *                 , foreign_context_address         , (control_t c
 PLX_ASIS(predicate_t            , foreign_context_predicate       , (control_t c), (c))
 
 PLX_VOID(void                   , register_extensions             , (const PL_extension *e), (e))
-PLX_VOID(void                   , register_extensions_in_module   , (const char *module, const PL_extension *e), (module, e));
+PLX_VOID(void                   , register_extensions_in_module   , (const char *module, const PL_extension *e), (module, e))
 
 // (skipped):: int PL_register_foreign(const char *name, int arity, pl_function_t func, int flags, ...);
 // (skipped):: int PL_register_foreign_in_module( const char *module , const char *name, int arity, pl_function_t func, int flags, ...);
@@ -124,7 +124,7 @@ PLX_VOID(void                    , clear_exception                 , (), ())
 // TODO: document PL_yielded()
 PLX_ASIS(term_t                  , yielded                         , (qid_t qid), (qid))
 PLX_EXCE(int                     , assert                          , (term_t term, module_t m, int flags), (term, m, flags))
-PLX_WRAP(term_t                  , new_term_refs                   , (int n), (n))
+PLX_WRAP(term_t                  , new_term_refs                   , (size_t n), (n))
 PLX_WRAP(term_t                  , new_term_ref                    , (), ())
 PLX_WRAP(term_t                  , copy_term_ref                   , (term_t from), (from))
 PLX_VOID(void                    , reset_term_refs                 , (term_t r), (r))
@@ -348,7 +348,7 @@ PLX_WRAP(atom_t                  , new_blob                        , (void *blob
 PLX_EXCE(int                     , put_blob                        , (term_t t, void *blob, size_t len, PL_blob_t *type), (t, blob, len, type))
 PLX_WRAP(int                     , get_blob                        , (term_t t, void **blob, size_t *len, PL_blob_t **type), (t, blob, len, type))
 PLX_ASIS(void*                   , blob_data                       , (atom_t a, size_t *len, struct PL_blob_t **type), (a, len, type))
-PLX_ASIS(int                     , free_blob                       , (atom_t blob), (blob));
+PLX_ASIS(int                     , free_blob                       , (atom_t blob), (blob))
 // Should not call PL_register_blob_type, so it's not defined:
 // PLX_VOID(void                 , register_blob_type              , (PL_blob_t *type), (type))
 PLX_ASIS(PL_blob_t*              , find_blob_type                  , (const char* name), (name))
@@ -503,8 +503,8 @@ PLX_ASIS(int                     , raise                           , (int sig),
 PLX_ASIS(int                     , handle_signals                  , (), ())
 PLX_ASIS(int                     , get_signum_ex                   , (term_t sig, int *n), (sig, n))
 // (skipped):: int PL_action(int, ...);
-PLX_VOID(void                    , on_halt                         , (int (*f)(int, void *), void *closure), (f, closure));
-PLX_VOID(void                    , exit_hook                       , (int (*f)(int, void *), void *closure), (f, closure));
+PLX_VOID(void                    , on_halt                         , (int (*f)(int, void *), void *closure), (f, closure))
+PLX_VOID(void                    , exit_hook                       , (int (*f)(int, void *), void *closure), (f, closure))
 PLX_VOID(void                    , backtrace                       , (int depth, int flags), (depth, flags))
 PLX_ASIS(char *                  , backtrace_string                , (int depth, int flags), (depth, flags))
 PLX_ASIS(int                     , check_data                      , (term_t data), (data))
@@ -534,7 +534,7 @@ PLX_EXCE(LRESULT                 , win_message_proc                , (HWND hwnd,
 PLX_ASIS(PL_engine_t             , create_engine                   , (PL_thread_attr_t *attributes), (attributes))
 PLX_ASIS(int                     , set_engine                      , (PL_engine_t engine, PL_engine_t *old), (engine, old))
 PLX_ASIS(int                     , destroy_engine                  , (PL_engine_t engine), (engine))
-PLX_ASIS(hash_table_t            , new_hash_table                  , (int size, void (*free_symbol)(void *n, void *v)), (size, free_symbol))
+PLX_ASIS(hash_table_t            , new_hash_table                  , (int size, void (*free_symbol)(table_key_t n, table_value_t v)), (size, free_symbol))
 PLX_ASIS(int                     , register_profile_type           , (PL_prof_type_t *type), (type))
 PLX_ASIS(void*                   , prof_call                       , (void *handle, PL_prof_type_t *type), (handle, type))
 PLX_VOID(void                    , prof_exit                       , (void *node), (node))
