@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1999-2023, University of Amsterdam
+    Copyright (c)  1999-2024, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
@@ -36,6 +36,7 @@
 */
 
 #include "pl-incl.h"
+#include "os/pl-cstack.h"
 
 #ifndef PL_THREAD_H_DEFINED
 #define PL_THREAD_H_DEFINED
@@ -113,8 +114,7 @@ typedef struct _PL_thread_info_t
   unsigned	    c_stack_low   : 1;	/* TRUE: Signalled low C stack */
   unsigned	    joined_by_creator : 1;
   int		    joining_by;		/* TID of joining thread */
-  void		   *c_stack_base;	/* Base of the (C-) stack */
-  size_t	    c_stack_size;	/* system (C-) stack */
+  c_stack_info	   *c_stack;
   size_t	    stack_limit;	/* Stack sizes */
   size_t	    table_space;	/* Max size for local tables */
   rc_cancel	    (*cancel)(int id);	/* cancel function */
@@ -178,7 +178,7 @@ typedef struct message_queue
 { struct thread_message   *head;	/* Head of message queue */
   struct thread_message   *tail;	/* Tail of message queue */
   uint64_t	       sequence_next;	/* next for sequence id */
-  word		       id;		/* Id of the queue */
+  atom_t	       id;		/* Id of the queue */
   size_t	       size;		/* # terms in queue */
   size_t	       max_size;	/* Max # terms in queue */
   int		       waiting;		/* # waiting threads */
@@ -510,11 +510,17 @@ static inline	PL_local_data_t *acquire_ldata(DECL_LD PL_thread_info_t *info);
 		 *	    FUNCTIONS		*
 		 *******************************/
 
+#ifndef __WINDOWS__
+#define HAVE_PTHREAD_EXIT 1
+#endif
+
 int		exitPrologThreads(void);
 bool		aliasThread(int tid, atom_t type, atom_t name);
-word		pl_thread_create(term_t goal, term_t id,
+foreign_t	pl_thread_create(term_t goal, term_t id,
 				 term_t options);
-word		pl_thread_exit(term_t retcode);
+#if HAVE_PTHREAD_EXIT
+foreign_t	pl_thread_exit(term_t retcode);
+#endif
 
 foreign_t	pl_thread_at_exit(term_t goal);
 int		PL_thread_self(void);
